//////////////////////////////////////////////////////////////////////////////
// persis.h
//
// persistent data header for the matching engine
//////////////////////////////////////////////////////////////////////////////

#ifndef __PERSIS_H__
#define __PERSIS_H__

#include "flddescr.h"

#ifdef DGIAPP
#define PERSIS_BLOCKS_IN_BUFFER        3  // DGI uses more than one block size for persis buffer.
#else
#define PERSIS_BLOCKS_IN_BUFFER        1  // DME uses one block in persis buffer.
#endif

#define PERSIS_BUFFER_MAX_SIZE         MESSAGE_TRANS_RECORD_SIZE*PERSIS_BLOCKS_IN_BUFFER

#define MESSAGE_HEADER_ALLOCATED_SIZE  150
#define MESSAGE_EXTRA_ALLOCATED_SIZE   100

// Default versions used for persistent files in DME.
#define PERSIS_FIRST_VERSION              100
#define PERSIS_VERSION_CRC32              101
#define PERSIS_VERSION_ENV_CRC32          102
#define PERSIS_VERSION_FACTRIESGO         103
#define PERSIS_VERSION_PENDCOBRANZA       104
#define PERSIS_VERSION_INDICATIVES        105
#define PERSIS_VERSION_CUPOSXDIA          106
#define PERSIS_VERSION_PRIMARYUSER        107
#define PERSIS_VERSION_CONFCUPOS          108

// Codes shared by all subclasses of CRecord_Data
#define PERSIS_RECORDDATA_VERSION               1
#define PERSIS_RECORDDATA_CRC32                 21
#define PERSIS_RECORDDATA_ENVIRONMENT           22
#define PERSIS_RECORDDATA_COMPLETE_BUFFER       10
#define PERSIS_RECORDDATA_END_OF_RECORD         11


#define MUST_SAVE_ALL_FILE_AGAIN 2

typedef void (*CallBackFunction_p)(char *error_text, int error_level);

void PERSIS_SetErrFunction (CallBackFunction_p cb);
void PERSIS_Report_Error (int error_level, const char *text_format, ...);

class CRecord_Data;
class CFile_Header;

class CFile_d
{
    FILE       *fp;
    Mutex_Type  m_lock;
public:
    CFile_Header *Last_CFile_Header;
    CFile_d ();
    ~CFile_d ();
    int open  (char *filename);
    int unlink (const char *filename);
    int rename (const char *old_name, const char *new_name);
    int close (void);
    int    seek  (unsigned int position);
    size_t write (void *data, unsigned d_len);
    size_t read  (void *data, unsigned d_len);
    int    flush (void);
};

// this class defines de basic read write operation 
// for a persistent data
class CPers_d
{
#define CPERS_DATA_HEAD (sizeof (unsigned int) + sizeof (unsigned int) + sizeof (int[4]))
  unsigned int  data_length;
  unsigned int   data_items;
  int            data_extra[4];
  char           data_buff[PERSIS_BUFFER_MAX_SIZE];
  char          *data_read;
  char          *data_write;
  unsigned int  data_read_len; 
  unsigned int  data_write_len; 
  unsigned int   data_count;
  int            m_big_to_lit_conv;

protected:
  unsigned int  new_position;
  CFile_d       *cf_p;

public:
  CPers_d (CFile_d *fp);
  virtual ~CPers_d ();

  // position, changes position
virtual void Set_Position (unsigned int np) { new_position = np; }
  // read  data from file
  int  Read_Data  (void);
  // write data to   file
  int  Write_Data (int max_write_len=0);

  int  Get_Next_Code (void);
  void Use_Big_End (void) {m_big_to_lit_conv = true; }
  void Not_Use_Big_End (void) {m_big_to_lit_conv = false; }
  int  Get_Big_End (void) {return m_big_to_lit_conv;}
  // get data
#define MEM_DATA  false
#define INT_DATA  true
  int  Get_Data (void *datap, unsigned int size, int Big_To_Lit_Conv = INT_DATA);
  // set data
  int  Set_Data  (int code, const void *datap, unsigned int size);
  int  Set_Char  (int code, const void *datap) {return Set_Data (code, datap, sizeof (char));}
  int  Set_Short (int code, const void *datap) {return Set_Data (code, datap, sizeof (short));}
  int  Set_UShort(int code, const void *datap) {return Set_Data (code, datap, sizeof (unsigned short));}
  int  Set_Int   (int code, const void *datap) {return Set_Data (code, datap, sizeof (int));}
  int  Set_UInt  (int code, const void *datap) {return Set_Data (code, datap, sizeof (unsigned int));}
  int  Set_Long  (int code, const void *datap) {return Set_Data (code, datap, sizeof (long));}
  int  Set_ULong (int code, const void *datap) {return Set_Data (code, datap, sizeof (unsigned int));}
  int  Set_Bool  (int code, const void *datap) {return Set_Data (code, datap, sizeof (bool));}
  int  Set_Calc  (int code, const void *datap);

  virtual int Serialize         (void) = 0;
  virtual int Print_Data        (void) = 0;
  virtual int Unserialize       (void);
  virtual int Unserialize_Content  (int n_code) {return 0;}
};


#define F_LIM_WRITE   1
#define F_LIM_READ    2
#define F_LIM_DEL     3
#define F_LIM_UNDEL   4
#define F_LIM_RLENGTH 5

class CFile_Header  : public CPers_d
{
public:

  unsigned int saved_file_code;
  unsigned int loaded_file_code;
  unsigned int total_number_of_records;
  unsigned int last_of_erased_records;
  unsigned int total_of_erased_records;
  unsigned int max_number_of_records;
  unsigned int record_data_length;
  char          date_reference[8];

  CFile_Header (unsigned int file_code, CFile_d *cf);
  void Clean (void);
  int Serialize   (void);
  int Print_Data  (void);
  int Unserialize_Content  (int n_code);
  int Review_Limit (unsigned int number, int number_type);
};

class CRecord_Header : public CPers_d
{
public:
  int           deleted;
  unsigned int record_length;

  CRecord_Header (CFile_d *cf);
  int Serialize   (void);
  int Print_Data  (void);
  int Unserialize_Content (int n_code);
  // friend class CRecord_Data;
};

#define NO_NEW_VERSION              0
#define NEW_VERSION_DETECTED        1
#define NEW_VERSION_UPDATE_FORCED   2


class CRecord_Data   : public CPers_d
{
  unsigned int   rec_data_saved;

  unsigned int   cur_record_num;
  CRecord_Header *header;
  unsigned int   rec_data_length;

protected:
  int             version;
  int             m_version_in_file;
  int             new_version_detected;
  void           *rec_data_ptr;
  void           *rec_extra_ptr;
  const void     *sv_rec_data_ptr;
  const void     *sv_rec_extra_ptr;
  unsigned int   rec_size_defined;

public:
  CRecord_Data (CFile_d *cf);
  void Set_Current_Version(int cur_version);
  void Set_Header  (CRecord_Header *h, 
                    unsigned int size_rec_length);
  int  Save_Header (unsigned int np);
  int  Serialize   (void);
  int  Serialize_Version      (void);
  int  Serialize_End_Of_Record(void);
  int  Print_Data  (void);
  int  Save_Record  (unsigned int np, const void *rec_data, const void *rec_extra=NULL);
  int  Load_Record  (unsigned int np, void *rec_data, void *rec_extra=NULL);
  int  Get_Version  (void) { return version; }

virtual int  Serialize_Content      (void);
virtual int  Unserialize_Content    (int n_code);
};

class CRecord_Crc32_Data   : public CRecord_Data
{
public:
  CRecord_Crc32_Data (CFile_d *cf);
  int  Serialize_Content    (void);
  int  Unserialize_Content  (int n_code);
};

class CInstNum_Record : public CRecord_Crc32_Data
{
public:

  CInstNum_Record (CFile_d *cf);
};

class CBranchNum_Record : public CRecord_Crc32_Data
{
public:

  CBranchNum_Record (CFile_d *cf);
};

class CGTPUser_Record : public CRecord_Crc32_Data
{
public:

  CGTPUser_Record (CFile_d *cf);
};

class CGTCity_Record : public CRecord_Crc32_Data
{
public:

  CGTCity_Record (CFile_d *cf);
};
class CGTCountry_Record : public CRecord_Crc32_Data
{
public:

  CGTCountry_Record (CFile_d *cf);
};
class CGTMoney_Record : public CRecord_Crc32_Data
{
public:

  CGTMoney_Record (CFile_d *cf);
};
class CGTInsperm_Record : public CRecord_Crc32_Data
{
public:

  CGTInsperm_Record (CFile_d *cf);
};
class CGTSuperGrp_Record : public CRecord_Crc32_Data
{
public:

  CGTSuperGrp_Record (CFile_d *cf);
};
class CGTVoiceGrp_Record : public CRecord_Crc32_Data
{
public:

  CGTVoiceGrp_Record (CFile_d *cf);
};
class CGTEmisores_Record : public CRecord_Crc32_Data
{
public:

  CGTEmisores_Record (CFile_d *cf);
};
class CGTComitent_Record : public CRecord_Crc32_Data
{
public:

  CGTComitent_Record (CFile_d *cf);
};

class CGTFiduciar_Record : public CRecord_Crc32_Data
{
public:

  CGTFiduciar_Record (CFile_d *cf);
};

class CGTEmployee_Record : public CRecord_Crc32_Data
{
public:

  CGTEmployee_Record (CFile_d *cf);
};

class CGTEsquema_Record : public CRecord_Crc32_Data
{
public:

  CGTEsquema_Record (CFile_d *cf);
};

class CGTSubComitent_Record : public CRecord_Crc32_Data
{
public:

  CGTSubComitent_Record (CFile_d *cf);
};

class CCrConf_Record : public CRecord_Data
{
public:

  CCrConf_Record (CFile_d *cf);
};

class CCrPoli_Record : public CRecord_Data
{
public:

  CCrPoli_Record (CFile_d *cf);
};

class CGtCuposG_Record : public CRecord_Data
{
public:

  CGtCuposG_Record (CFile_d *cf);
};

class CGtNmchkPlz_Record : public CRecord_Data
{
public:

  CGtNmchkPlz_Record (CFile_d *cf);
};

class CGtCreditAdm_Record : public CRecord_Data
{
public:

  CGtCreditAdm_Record (CFile_d *cf);
};

class CCredit_Record : public CRecord_Data
{
public:

  CCredit_Record (CFile_d *cf);
};

class CGTBridge_Record : public CRecord_Data
{
public:

  CGTBridge_Record (CFile_d *cf);
};

class CGTCounterBridge_Record : public CRecord_Data
{
public:

  CGTCounterBridge_Record (CFile_d *cf);
};

class CGTSpot_Record : public CRecord_Data
{
public:

  CGTSpot_Record (CFile_d *cf);
};

class CGTOvernight_Record : public CRecord_Data
{
public:

  CGTOvernight_Record (CFile_d *cf);
};

class CGTCashRate_Record : public CRecord_Data
{
public:

  CGTCashRate_Record (CFile_d *cf);
};

class CGTIndicatives_Record : public CRecord_Data
{
public:

  CGTIndicatives_Record (CFile_d *cf);
  int Unserialize_Content(int n_code);

};

class CGTCuposL_Record : public CRecord_Crc32_Data
{
public:

  CGTCuposL_Record (CFile_d *cf);
};

class CGTCuposXPL_Record : public CRecord_Crc32_Data
{
public:

  CGTCuposXPL_Record (CFile_d *cf);
};

class CGTCuposXDIA_Record : public CRecord_Crc32_Data
{
public:

  CGTCuposXDIA_Record (CFile_d *cf);
#ifdef DMEAPP
  int Unserialize_Content(int n_code);
#endif
};

class CGTPendCobranzas_Record : public CRecord_Crc32_Data
{
public:

  CGTPendCobranzas_Record (CFile_d *cf);
#ifdef DMEAPP
  int Unserialize_Content(int n_code);
#endif
};
class CGTFactRiesgo_Record : public CRecord_Crc32_Data
{
public:

  CGTFactRiesgo_Record (CFile_d *cf);
};
class CGTLimInstit_Record : public CRecord_Crc32_Data
{
public:

  CGTLimInstit_Record (CFile_d *cf);
};
class CGTConfCupos_Record : public CRecord_Crc32_Data
{
public:
#ifdef DMEAPP
  int Unserialize_Content(int n_code);
#endif
  CGTConfCupos_Record (CFile_d *cf);
};


class CGTConfirmPend_Record : public CRecord_Data
{
public:
  CGTConfirmPend_Record (CFile_d *cf);
};

class CGTIndicativesMark_Dme_Record : public CRecord_Data
{
public:
  CGTIndicativesMark_Dme_Record (CFile_d *cf);
};

class CGTGenData_Record : public CRecord_Data
{
public:
  CGTGenData_Record (CFile_d *cf);
};

class CGTComparCupos_Record : public CRecord_Crc32_Data
{
public:

  CGTComparCupos_Record (CFile_d *cf);
};


#define PERS_MAX_FILENAME 100

// DME persistent files codes.
#define PERS_INSTITNUM_CODE         101
#define PERS_BRANCHNUM_CODE         102
#define PERS_CRCONF_CODE            103
#define PERS_CRPOLICY_CODE          104
#define PERS_GTCUPOSG_CODE          105
#define PERS_GTNMCHKPLZ_CODE        106
#define PERS_GTCREDITADM_CODE       107
#define PERS_BRIDGE_CODE             80
#define PERS_SPOT_CODE               81
#define PERS_OVERNIGHT_CODE          82
#define PERS_CASHRATE_CODE           83
#define PERS_INDICATIVES_CODE        84
#define PERS_COUNTERBRIDGE_CODE      85

#define PERS_CUPOSL_CODE             108
#define PERS_PENDCOBRANZAS_CODE      110 
#define PERS_LIMINSTIT_CODE          111
#define PERS_FACTRIESGO_CODE         112 
#define PERS_CONFIRMPEND_CODE        113
#define PERS_CONFCUPOS_CODE          114
#define PERS_INDICATIVESDMEMARK_CODE 115
#define PERS_GENDATA_CODE            116
#define PERS_CUPOSXPL_CODE           117
#define PERS_CUPOSXDIA_CODE          118

#define PERS_CITY_CODE               127
#define PERS_COUNTRY_CODE            128
#define PERS_PYUSER_CODE             129
#define PERS_MONEY_CODE              130
#define PERS_INSPERM_CODE            131   
#define PERS_SUPERGROUP_CODE         133
#define PERS_VOICEGROUP_CODE         134
#define PERS_EMISORES_CODE           135
#define PERS_COMITENT_CODE           136   
#define PERS_FIDUCIAR_CODE           137
#define PERS_COMPART_CUPO_CODE       138
#define PERS_EMPLOYEE_CODE           139
#define PERS_ESQUEMA_CODE            140  
#define PERS_SUBCOMITENT_CODE        141

#define PERS_TEST_CODE              999
#define PERS_INIT_DELETED_NODES     200



class CPersFile
{
public:
  char            m_file_name[PERS_MAX_FILENAME+1];
  unsigned int   m_rec_size;
  int             m_working;
  unsigned int   m_file_code;
  unsigned int   m_local_number;
  unsigned int  *m_max_number_p;
  int             m_deleted_array_size;
  int             m_cur_deleted_array;
  int             m_save_file_again;

  CFile_d         m_h;
  CFile_Header    m_header;
  CRecord_Header  m_record_header;
  CRecord_Data   *m_record_p;
  CvArray <unsigned int > m_deleted_array;


   CPersFile (unsigned int file_code, const char *file_name, unsigned int rec_size);
  virtual ~CPersFile (void);
  unsigned int Set_File_Code (unsigned int file_code);

#define OPEN_RENAME_FILE 1
#define DO_NOT_RENAME    0
  int  Open  (int rename_original=DO_NOT_RENAME);
  int  Load_Record   (unsigned int rec_num, void *rec_data, void *rec_extra=NULL);
  int  Save_Record   (unsigned int rec_num, const void *rec_data, const void *rec_extra=NULL);
  int  Load_Next_Record   (unsigned int *rec_num, void *rec_data, void *rec_extra=NULL);
  int  Check_Forced_Save  (void);
  int  Delete_Record      (unsigned int rec_num);
  unsigned int Get_Next_Free (void);
  void Print_File (void);
  int  Close (void);
  void Set_Date_Reference (char *date);
  void Get_Date_Reference (char *date);

  virtual bool Get_Persis_Record(void);
};

extern const char *PERSIS_FILE_ERROR;

int PERSIS_Test   (char *test_file);
int PERSIS_Export (const char *pers_file_name, const char *output_file_name);
#endif // __PERSIS_H__

//////////////////////////////////  end of file  //////////////////////////////////////////
